unit FormMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComObj, ShlObj, ActiveX, ExtCtrls, MyCOMEventsHookUnit,
  CommonClient_TLB, spsdksw_TLB, SpRecordLibTypesW, CheckLst;

type
  TMainForm = class(TForm)
    LabelServer: TLabel;
    cbServer: TComboBox;
    btnSelectServer: TButton;
    btnOpen: TButton;
    btnConnect: TButton;
    btnDisconnect: TButton;
    btnClose: TButton;
    MemoInfo: TMemo;
    btnGetLibVersion: TButton;
    btnGetSpRecordVersion: TButton;
    btnGetChannelNames: TButton;
    btnGetDeviceNames: TButton;
    cbHardChannelNames: TCheckBox;
    cbUseCOMEvents: TCheckBox;
    cbChannels: TComboBox;
    cbDevices: TComboBox;
    btnGetDeviceChannelNames: TButton;
    LabelDeviceNames: TLabel;
    LabelChannelNames: TLabel;
    btnGetDeviceSupport: TButton;
    btnClearLog: TButton;
    TimerMsg: TTimer;
    btnGetChannelSupport: TButton;
    btnGetChannelNamesEx: TButton;
    cbReadyToCall: TCheckBox;
    clbSupports: TCheckListBox;
    procedure btnOpenClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure btnGetLibVersionClick(Sender: TObject);
    procedure btnGetSpRecordVersionClick(Sender: TObject);
    procedure btnGetChannelNamesClick(Sender: TObject);
    procedure btnGetDeviceNamesClick(Sender: TObject);
    procedure btnGetDeviceChannelNamesClick(Sender: TObject);
    procedure btnGetDeviceSupportClick(Sender: TObject);
    procedure btnClearLogClick(Sender: TObject);
    procedure btnSelectServerClick(Sender: TObject);
    procedure TimerMsgTimer(Sender: TObject);
    procedure btnGetChannelSupportClick(Sender: TObject);
    procedure cbUseCOMEventsClick(Sender: TObject);
    procedure btnGetChannelNamesExClick(Sender: TObject);
  private
    FMain: ISpRecordClientWDisp;
    FEventsHook: IMyCOMEventsHook;
    FCP: IConnectionPoint;
    FCookie: Longint;
    procedure ShowSDKError(Error: Integer);
    procedure ShowSDKMsg(const Msg: OleVariant);
    procedure RefreshCOMEvents;
    procedure OnMessage(const Msg: OleVariant);
    procedure RefreshEnables;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

const
  ChannelFlag: array[Boolean] of Integer = (SPR_CHANNEL_USERNAME, SPR_CHANNEL_HARDNAME);

procedure TMainForm.ShowSDKError(Error: Integer);
  var eStr: String;
begin
  if Error and $80000000 = 0 then
  begin
    eStr := 'Operation complited.';
    if Error <> SPR_S_OK then eStr := eStr + ' Result: 0x' + IntToHex(Error, 8);
  end else
    eStr := 'Operation failed. Error: 0x' + IntToHex(Error, 8);
  MemoInfo.Lines.Add('');
  MemoInfo.Lines.Add(eStr);
  MemoInfo.Lines.Add('');
end;

procedure TMainForm.ShowSDKMsg(const Msg: OleVariant);
  var lpMsg: PMSGHeader;
        Str: String;
begin
  lpMsg := VarArrayLock(Msg);
  try
    Str := DateTimeToStr(lpMsg.MsgTime) + ' [0x' + IntToHex(lpMsg.dwMsg, 8) + ']: ';
    case lpMsg.dwMsg of
      SPR_MSG_RECORD_START:
        with PMSGRecordStart(lpMsg)^ do
          Str := Str + cbChannelName + ', Type: ' + IntToStr(dwRecordType) +
            ', PhoneFrom: ' + cbPhoneFrom + ', PhoneTo: ' + cbPhoneTo +
            ', FileName: ' + cbFileName;
      SPR_MSG_RING:
        with PMSGRing(lpMsg)^ do
          Str := Str + cbChannelName + ', RingCount: ' + IntToStr(dwRingCount);
      SPR_MSG_ANI_DETECT:
        with PMSGPhoneNumber(lpMsg)^ do
          Str := Str + cbChannelName + ', ANI: ' + cbPhoneNumber;
      SPR_MSG_RECORD_TYPE:
        with PMSGRecordType(lpMsg)^ do
          Str := Str + cbChannelName + ', RingType: ' + IntToStr(dwRecordType);
      SPR_MSG_DIALLED_PHONE:
        with PMSGPhoneNumber(lpMsg)^ do
          Str := Str + cbChannelName + ', NewOutPhone: ' + cbPhoneNumber;
      SPR_MSG_MISSED_PHONE:
        with PMSGMissedPhone(lpMsg)^ do
          Str := Str + cbChannelName + ', MissedPhoneFrom: ' + cbPhoneFrom +
            ', MissedPhoneTo: ' + cbPhoneTo;
      SPR_MSG_RECORD_STOP:
        with PMSGRecordStop(lpMsg)^ do
          Str := Str + cbChannelName + ', RingType: ' + IntToStr(dwRecordType) +
            ', PhoneFrom: ' + cbPhoneFrom + ', cbPhoneTo: ' + cbPhoneTo +
            ', FileName: ' + cbFileName + ', Leng: ' + IntToStr(dwLeng) +
            ', NormalStop: ' + BoolToStr(NormalStop, True) +
            ', bCompress: ' + BoolToStr(bCompress, True);
      SPR_MSG_CHANGE_CHANNEL_NAME:
        with PMSGChangeChannelName(lpMsg)^ do
          Str := Str + cbOldChannelName + ', NewName: ' + cbNewChannelName;
      SPR_MSG_TEXT:
        with PMSGText(lpMsg)^ do Str := Str + PChar(@cbText);
      SPR_MSG_DESTROY_CONNECTION:
        with PMSGDestroyConnection(lpMsg)^ do
          Str := Str + 'DestroyReason: ' + IntToStr(dwDestroyReason);
      SPR_MSG_DEVICE_CONNECTED:
        with PMSGDeviceName(lpMsg)^ do
          Str := Str + 'DeviceConnect: ' + cbDeviceName;
      SPR_MSG_DEVICE_DISCONNECTED:
        with PMSGDeviceDisconnect(lpMsg)^ do
          Str := Str + 'DeviceDisonnect: ' + cbDeviceName + ', Reason: ' + IntToStr(dsDisconnectReason);
      SPR_MSG_CONVERSATION_BEGIN:
        with PMSGChannelName(lpMsg)^ do
          Str := Str + cbChannelName + ', Conversation begin';
      SPR_MSG_CONVERSATION_END:
        with PMSGChannelName(lpMsg)^ do
          Str := Str + cbChannelName + ', Conversation end';
      SPR_MSG_POWERSUSPEND: Str := Str + 'Power Suspend';
      SPR_MSG_COMPRESSING_FINISHED:
        Str := Str + 'Compressing finished: ' + PMSGCompressingFinished(lpMsg).cbFileName;
      SPR_MSG_FAX_DETECTED:
        with PMSGChannelName(lpMsg)^ do
          Str := Str + cbChannelName + ', Fax detected';
      else
        Str := Str + 'Unknow message';
    end;
  finally
    VarArrayUnlock(Msg);
  end;
  MemoInfo.Lines.Add(Str);
end;

procedure TMainForm.OnMessage(const Msg: OleVariant);
begin
  ShowSDKMsg(Msg);
end;

procedure TMainForm.RefreshCOMEvents;
  var CPC: IConnectionPointContainer;
    cHook: TMyCOMEventsHook;
begin
  if FMain <> nil then
  if cbUseCOMEvents.Checked then
  begin
    cHook := TMyCOMEventsHook.Create;
    cHook.OnHook := OnMessage;
    FEventsHook := cHook as IMyCOMEventsHook;

    CPC := FMain as IConnectionPointContainer;
    OleCheck(CPC.FindConnectionPoint(DIID_ISpRecordClientWEvents, FCP));
    OleCheck(FCP.Advise(FEventsHook, FCookie));

    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Events seted.');
    MemoInfo.Lines.Add('');
  end else
  begin
    if Assigned(FCP) then
      OleCheck(FCP.Unadvise(FCookie));

    FCP := nil;
    FEventsHook := nil;

    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Events unseted.');
    MemoInfo.Lines.Add('');
  end;
  TimerMsg.Enabled := (FMain <> nil) and (FEventsHook = nil);
end;

procedure TMainForm.RefreshEnables;
begin
  btnOpen.Enabled := FMain = nil;
  btnConnect.Enabled := (FMain <> nil) and not FMain.Connected;
  btnDisconnect.Enabled := (FMain <> nil) and FMain.Connected;
  btnClose.Enabled := FMain <> nil;

  btnGetLibVersion.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetSpRecordVersion.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetDeviceNames.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetChannelNames.Enabled := (FMain <> nil) and FMain.Connected;
  LabelDeviceNames.Enabled := (FMain <> nil) and FMain.Connected;
  cbDevices.Enabled := (FMain <> nil) and FMain.Connected;
  LabelChannelNames.Enabled := (FMain <> nil) and FMain.Connected;
  cbChannels.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetDeviceSupport.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetDeviceChannelNames.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetChannelSupport.Enabled := (FMain <> nil) and FMain.Connected;
  btnGetChannelNamesEx.Enabled := (FMain <> nil) and FMain.Connected;
end;

procedure TMainForm.TimerMsgTimer(Sender: TObject);
  var Msg: OleVariant;
begin
 if FMain <> nil then
 try
   while FMain.PeekMessage(Msg) = SPR_S_OK do
     ShowSDKMsg(Msg);
 except
   btnClose.Click;
   raise;
 end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  MemoInfo.Clear;
  RefreshEnables;
end;

procedure TMainForm.btnClearLogClick(Sender: TObject);
begin
  MemoInfo.Clear;
end;

procedure TMainForm.btnSelectServerClick(Sender: TObject);
  var
    WindowList: Pointer;
    BrowseInfo: TBrowseInfo;
    Buffer: PChar;
    RootItemIDList, ItemIDList: PItemIDList;
    ShellMalloc: IMalloc;
begin
  FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
  if (ShGetMalloc(ShellMalloc) = S_OK) and (ShellMalloc <> nil) then
  begin
    Buffer := ShellMalloc.Alloc(MAX_PATH);
    try
      if SHGetSpecialFolderLocation(Application.Handle, CSIDL_NETWORK, RootItemIDList) <> NOERROR then
        RootItemIDList := nil;
      with BrowseInfo do
      begin
        hwndOwner := Application.Handle;
        pidlRoot := RootItemIDList;
        pszDisplayName := Buffer;
        lpszTitle := 'Select computer name:';
        ulFlags := BIF_BROWSEFORCOMPUTER;
      end;
      WindowList := DisableTaskWindows(0);
      try
        ItemIDList := ShBrowseForFolder(BrowseInfo);
      finally
        EnableTaskWindows(WindowList);
      end;
      if ItemIDList <> nil then
      begin
        ShellMalloc.Free(ItemIDList);
        cbServer.Text := Buffer;
      end;
    finally
      ShellMalloc.Free(Buffer);
    end;
  end;
end;

procedure TMainForm.cbUseCOMEventsClick(Sender: TObject);
begin
  RefreshCOMEvents;
end;

procedure TMainForm.btnOpenClick(Sender: TObject);
  var Server: String;
         Obj: IUnknown;
begin
  Server := cbServer.Text;
  if Server = '' then Obj := CreateComObject(CLASS_SpRecordClientW)
    else Obj := CreateRemoteComObject(Server, CLASS_SpRecordClientW);
  FMain := ISpRecordClientWDisp(Obj as IDispatch);
  RefreshCOMEvents;
  RefreshEnables;
end;

procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  TimerMsg.Enabled := False;
  try
    if Assigned(FCP) then OleCheck(FCP.Unadvise(FCookie));
  finally
    FCP := nil;
    FEventsHook := nil;
    FMain := nil;
    cbDevices.Clear;
    cbChannels.Clear;
    RefreshEnables;
  end;
end;

procedure TMainForm.btnConnectClick(Sender: TObject);
  var eResult: Integer;
begin
  eResult := FMain.Connect(0, '', '');
  ShowSDKError(eResult);
  RefreshEnables;
end;

procedure TMainForm.btnDisconnectClick(Sender: TObject);
  var eResult: Integer;
begin
  eResult := FMain.Disconnect;
  ShowSDKError(eResult);
  cbDevices.Clear;
  cbChannels.Clear;
  RefreshEnables;
end;

procedure TMainForm.btnGetLibVersionClick(Sender: TObject);
  var Version: Integer;
begin
  Version := FMain.GetLibVersion;
  MemoInfo.Lines.Add('');
  MemoInfo.Lines.Add('SDK DLL Version: ' + IntToStr(Version shr 16) + '.' + IntToStr(Version and $FFFF));
  MemoInfo.Lines.Add('');
end;

procedure TMainForm.btnGetSpRecordVersionClick(Sender: TObject);
  var Version: Integer;
      eResult: Integer;
begin
  eResult := FMain.GetSpRecordVersion(Version);
  if eResult = SPR_S_OK then
  begin
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('SpRecord Version: ' + IntToStr(Version shr 16) + '.' + IntToStr(Version and $FFFF));
    MemoInfo.Lines.Add('');
  end else
    ShowSDKError(eResult);
end;

procedure TMainForm.btnGetDeviceNamesClick(Sender: TObject);
  var Names: WideString;
    eResult: Integer;
     lNames: TStrings;
begin
  eResult := FMain.GetDeviceNames(Names);
  if eResult = SPR_S_OK then
  begin
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Device Names:');
    lNames := TStringList.Create;
    try
      lNames.LineBreak := '|';
      lNames.Text := Names;
      MemoInfo.Lines.AddStrings(lNames);
      cbDevices.Items.Assign(lNames);
    finally
      lNames.Free;
    end;
    MemoInfo.Lines.Add('');
    if cbDevices.Items.Count > 0 then cbDevices.ItemIndex := 0;
  end else
    ShowSDKError(eResult);
end;

procedure TMainForm.btnGetChannelNamesClick(Sender: TObject);
  var Names: WideString;
    eResult: Integer;
     lNames: TStrings;
begin
  eResult := FMain.GetChannelNames(ChannelFlag[cbHardChannelNames.Checked], Names);
  if eResult = SPR_S_OK then
  begin
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Channel names:');
    lNames := TStringList.Create;
    try
      lNames.LineBreak := '|';
      lNames.Text := Names;
      MemoInfo.Lines.AddStrings(lNames);
      cbChannels.Items.Assign(lNames);
    finally
      lNames.Free;
    end;
    MemoInfo.Lines.Add('');
    if cbChannels.Items.Count > 0 then cbChannels.ItemIndex := 0;
  end else
    ShowSDKError(eResult);
end;

procedure TMainForm.btnGetChannelNamesExClick(Sender: TObject);
  var Names: WideString;
    eResult: Integer;
     lNames: TStrings;
    Support: Integer;
          i: Integer;
begin
  Support := 0;
  for i := 0 to clbSupports.Items.Count - 1 do
    if clbSupports.Checked[i] then
      Support := Support or 1 shl i;
  eResult := FMain.GetChannelNamesEx(ChannelFlag[cbHardChannelNames.Checked],
    Support, cbReadyToCall.Checked, Names);
  if eResult = SPR_S_OK then
  begin
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Channel names:');
    lNames := TStringList.Create;
    try
      lNames.LineBreak := '|';
      lNames.Text := Names;
      MemoInfo.Lines.AddStrings(lNames);
      cbChannels.Items.Assign(lNames);
    finally
      lNames.Free;
    end;
    MemoInfo.Lines.Add('');
    if cbChannels.Items.Count > 0 then cbChannels.ItemIndex := 0;
  end else
    ShowSDKError(eResult);
end;

procedure TMainForm.btnGetDeviceSupportClick(Sender: TObject);
  var Support: Integer;
      eResult: Integer;
       Device: String;
          Str: String;
begin
  Device := cbDevices.Text;
  eResult := FMain.GetDeviceSupport(Device, Support);
  if eResult = SPR_S_OK then
  begin
    Str := '';
    if Support and SPR_DEVICE_SUPPORT_LINESTATE <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'Line State';
    end;
    if Support and SPR_DEVICE_SUPPORT_VOICEMAIL <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'VoiceMail';
    end;
    if Support and SPR_DEVICE_SUPPORT_ISDNBRI <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'ISDN BRI';
    end;
    if Support and SPR_DEVICE_SUPPORT_ISDN <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'ISDN';
    end;
    if Support and SPR_DEVICE_SUPPORT_DIGITAL <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'DIGITAL';
    end;
    if Str = '' then Str := 'none';
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Device ' + Device + ' support ' + Str);
    MemoInfo.Lines.Add('');
  end else
    ShowSDKError(eResult);
end;

procedure TMainForm.btnGetDeviceChannelNamesClick(Sender: TObject);
  var Names: WideString;
    eResult: Integer;
     lNames: TStrings;
     Device: String;
begin
  Device := cbDevices.Text;
  eResult := FMain.GetDeviceChannelNames(Device, ChannelFlag[cbHardChannelNames.Checked], Names);
  if eResult = SPR_S_OK then
  begin
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Channel names for ' + Device + ':');
    lNames := TStringList.Create;
    try
      lNames.LineBreak := '|';
      lNames.Text := Names;
      MemoInfo.Lines.AddStrings(lNames);
      cbChannels.Items.Assign(lNames);
    finally
      lNames.Free;
    end;
    MemoInfo.Lines.Add('');
    if cbChannels.Items.Count > 0 then cbChannels.ItemIndex := 0;
  end else
    ShowSDKError(eResult);
end;

procedure TMainForm.btnGetChannelSupportClick(Sender: TObject);
  var Support: Integer;
      eResult: Integer;
      Channel: String;
          Str: String;
begin
  Channel := cbChannels.Text;
  eResult := FMain.GetChannelSupport(Channel, ChannelFlag[cbHardChannelNames.Checked], Support);
  if eResult = SPR_S_OK then
  begin
    Str := '';
    if Support and SPR_DEVICE_SUPPORT_LINESTATE <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'Line State';
    end;
    if Support and SPR_DEVICE_SUPPORT_VOICEMAIL <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'VoiceMail';
    end;
    if Support and SPR_DEVICE_SUPPORT_ISDNBRI <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'ISDN BRI';
    end;
    if Support and SPR_DEVICE_SUPPORT_ISDN <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'ISDN';
    end;
    if Support and SPR_DEVICE_SUPPORT_DIGITAL <> 0 then
    begin
      if Str <> '' then Str := Str + ', ';
      Str := Str + 'DIGITAL';
    end;
    if Str = '' then Str := 'none';
    MemoInfo.Lines.Add('');
    MemoInfo.Lines.Add('Channel ' + Channel + ' support ' + Str);
    MemoInfo.Lines.Add('');
  end else
    ShowSDKError(eResult);
end;

end.
